/***********************************************************************
 *
 * linux/arch/arm/mach-s3c2410/mach-ln2410sbc.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * @History:
 * derived from linux/arch/arm/mach-s3c2410/mach-bast.c, written by
 * Ben Dooks <ben@simtec.co.uk>
 *
 ***********************************************************************/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/platform_device.h>

#include <linux/mmc/protocol.h>
#include <linux/mmc/host.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>
#include <asm/mach/mmc.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <asm/arch/regs-gpio.h>
#include <asm/arch/regs-serial.h>
#include <asm/arch/regs-clock.h>
#include <asm/arch/ln2410sbc-map.h>
#include <asm/arch/nand.h>
#include <asm/arch/fb.h>
#include <asm/arch/mmc.h>
#include <asm/arch/udc.h>
#include <asm/arch/ts.h>
#include <asm/arch/buttons.h>
#include <asm/arch/js_buttons.h>

#include <asm/plat-s3c24xx/clock.h>
#include <asm/plat-s3c24xx/devs.h>
#include <asm/plat-s3c24xx/cpu.h>
#include <asm/plat-s3c24xx/pm.h>

static struct map_desc ln2410sbc_iodesc[] __initdata = {
	[0] = {
		.virtual	= (unsigned long)LN2410SBC_VA_CS8900A_BASE,
		.pfn		= __phys_to_pfn(LN2410SBC_PA_CS8900A_BASE),
		.length		= SZ_4M,
		.type		= MT_DEVICE
	},
	[1] = {
		.virtual	= (unsigned long)LN2410SBC_VA_IDECMD,
		.pfn		= __phys_to_pfn(LN2410SBC_PA_IDECMD),
		.length		= SZ_1M,
		.type		= MT_DEVICE
	},
	[2] = {
		.virtual	= (unsigned long)LN2410SBC_VA_IDECTL,
		.pfn		= __phys_to_pfn(LN2410SBC_PA_IDECTL),
		.length		= SZ_1M,
		.type		= MT_DEVICE
	},
	[3] = {
		.virtual	= (unsigned long)LN2410SBC_VA_CF_IO_MEM,
		.pfn		= __phys_to_pfn(LN2410SBC_PA_CF_IO_MEM),
		.length		= SZ_1M,
		.type		= MT_DEVICE
	},
	[4] = {
		.virtual	= (unsigned long)LN2410SBC_VA_CF_ATTRIB_MEM,
		.pfn		= __phys_to_pfn(LN2410SBC_PA_CF_ATTRIB_MEM),
		.length		= SZ_1M,
		.type		= MT_DEVICE
	},
	[5] = {
		.virtual	= (unsigned long)LN2410SBC_VA_CF_COMM_MEM,
		.pfn		= __phys_to_pfn(LN2410SBC_PA_CF_COMM_MEM),
		.length		= SZ_1M,
		.type		= MT_DEVICE
	}
};
/*
static struct map_desc ln2410sbc_iodesc[] __initdata = {
   { (u32)LN2410SBC_VA_CS8900A_BASE, LN2410SBC_PA_CS8900A_BASE, SZ_4M, MT_DEVICE },
   { (u32)LN2410SBC_VA_IDECMD, LN2410SBC_PA_IDECMD, SZ_1M, MT_DEVICE },
   { (u32)LN2410SBC_VA_IDECTL, LN2410SBC_PA_IDECTL, SZ_1M, MT_DEVICE },
   { (u32)LN2410SBC_VA_CF_IO_MEM, LN2410SBC_PA_CF_IO_MEM, SZ_1M, MT_DEVICE },
   { (u32)LN2410SBC_VA_CF_ATTRIB_MEM, LN2410SBC_PA_CF_ATTRIB_MEM, SZ_1M, MT_DEVICE },
   { (u32)LN2410SBC_VA_CF_COMM_MEM, LN2410SBC_PA_CF_COMM_MEM, SZ_1M, MT_DEVICE },
};
*/
#define UCON S3C2410_UCON_DEFAULT
#define ULCON S3C2410_LCON_CS8 | S3C2410_LCON_PNONE | S3C2410_LCON_STOPB
#define UFCON S3C2410_UFCON_RXTRIG8 | S3C2410_UFCON_FIFOMODE

static struct s3c2410_uartcfg ln2410sbc_uartcfgs[] = {
	[0] = {
		.hwport	     = 0,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	},
	[1] = {
		.hwport	     = 1,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	},
	[2] = {
		.hwport	     = 2,
		.flags	     = 0,
		.ucon	     = UCON,
		.ulcon	     = ULCON,
		.ufcon	     = UFCON,
	}
};

static void ln2410sbc_pullup(enum s3c2410_udc_cmd_e cmd)
{
	printk(KERN_DEBUG "udc: pullup(%d)\n",cmd);
	switch (cmd)
	{
		case S3C2410_UDC_P_ENABLE :
			s3c2410_gpio_setpin(S3C2410_GPE3,0);
			break;
		case S3C2410_UDC_P_DISABLE :
	          	s3c2410_gpio_setpin(S3C2410_GPE3,1);
			break;
		case S3C2410_UDC_P_RESET :
			break;
		default: break;
	}
}

static struct s3c2410_udc_mach_info ln2410_udc_cfg __initdata = {
		.udc_command = ln2410sbc_pullup,
		.vbus_pin = 0,
};

static struct s3c2410_ts_mach_info ln2410_ts_cfg __initdata = {
		.delay = 10000,
		.presc = 49,
		.oversampling_shift = 2,
};


static struct s3c2410_button ln2410_butts[] = {
	DECLARE_S3C2410_BUTTON(S3C2410_GPF4, BTN_0, "Btn0", 0, 1, 1,1 ),
	DECLARE_S3C2410_BUTTON(S3C2410_GPF6, BTN_2, "Btn2", 0, 1, 1,1 ),
	DECLARE_S3C2410_BUTTON(S3C2410_GPF7, BTN_3, "Btn3", 0, 1, 1,1 ),
	DECLARE_S3C2410_BUTTON(S3C2410_GPG0, BTN_4, "Btn4", 0, 1, 1,1 ),
	DECLARE_S3C2410_BUTTON(S3C2410_GPF5, BTN_1, "Btn1", 0, 1, 1,1 ), 
};

static struct s3c2410_butt_mach_info ln2410_butt_cfg __initdata = {
	.buttons  = ln2410_butts,
	.size = ARRAY_SIZE(ln2410_butts),
};

static struct s3c2410_jsbutton ln2410_jsbutts[] = {
	DECLARE_S3C2410_JS_BUTTON(S3C2410_GPG1, 0, "S3C2410_jsbuttons LF",0),
	DECLARE_S3C2410_JS_BUTTON(S3C2410_GPG8, 1, "S3C2410_jsbuttons RT", 0),
	DECLARE_S3C2410_JS_BUTTON(S3C2410_GPG9,  2, "S3C2410_jsbuttons UP", 0),
	DECLARE_S3C2410_JS_BUTTON(S3C2410_GPG10, 3, "S3C2410_jsbuttons DN", 0),
	DECLARE_S3C2410_JS_BUTTON(S3C2410_GPG11, 4, "S3C2410_jsbuttons MID", 0),
};

static struct s3c2410_jsbutt_mach_info ln2410_jsbutt_cfg __initdata = {
	.buttons  = ln2410_jsbutts,
	.size = ARRAY_SIZE(ln2410_jsbutts),
};

struct mtd_partition ln2410_default_nand_part[] = {
	[0] = {
		.name	= "NAND partition 0 : Reserver",
		.offset	= 0,
		.size	= (1*SZ_1M)
	},
	[1] = {
		.name	= "NAND partition 1 : Kernel",
		.offset	= (1*SZ_1M),
		.size	= (3*SZ_1M) - (1*SZ_1M)
	},
	[2] = {
		.name	= "NAND partition 2 : Ramdisk",
		.offset	= (3*SZ_1M),
		.size	= (8*SZ_1M) - (3*SZ_1M)
	},
	[3] = {
		.name	= "NAND partition 3 : /mtd (YAFFS)",
		.offset	= (8*SZ_1M),
		.size	= (32*SZ_1M) - (8*SZ_1M)
	}
};

static int chip0_map[]	= { 0 };

static struct s3c2410_nand_set ln2410_nand_sets[] = {
	[0] = {
		.nr_chips	= 1,
		.nr_partitions	= ARRAY_SIZE(ln2410_default_nand_part),
		.name		= "chip0",
		.nr_map		= chip0_map,
		.partitions	= ln2410_default_nand_part
	}
};

static struct s3c2410_platform_nand ln2410_nand_info = {
	.tacls		= 15,
	.twrph0		= 30,
	.twrph1		= 30,
	.nr_sets	= ARRAY_SIZE(ln2410_nand_sets),
	.sets		= ln2410_nand_sets,
};

static struct s3c2410fb_mach_info ln2410sbc_lcdcfg __initdata = {
/*
 *  LP64 screen (6.4 inch, 640x480) setup
 */
#ifdef CONFIG_LN2410SBC_LP64
#define S3C2410_WIDTH    640
#define S3C2410_HEIGHT   480
//	.fixed_syncs=		1,
	.type=			S3C2410_LCDCON1_TFT,
	.regs={
		.lcdcon1=	S3C2410_LCDCON1_TFT16BPP \
                                | S3C2410_LCDCON1_TFT \
                                | S3C2410_LCDCON1_CLKVAL(0x1),

		.lcdcon2=	S3C2410_LCDCON2_VBPD(31) \
                                | S3C2410_LCDCON2_LINEVAL(S3C2410_HEIGHT-1) \
                                | S3C2410_LCDCON2_VFPD(10) \
				        | S3C2410_LCDCON2_VSPW(1),

		.lcdcon3=	S3C2410_LCDCON3_HBPD(47) \
                                | S3C2410_LCDCON3_HOZVAL(S3C2410_WIDTH-1) \
				        | S3C2410_LCDCON3_HFPD(15),

		.lcdcon4=	S3C2410_LCDCON4_MVAL(13) \
				        | S3C2410_LCDCON4_HSPW(95),

		.lcdcon5=	S3C2410_LCDCON5_FRM565 \
                                | S3C2410_LCDCON5_INVVLINE \
                                | S3C2410_LCDCON5_INVVFRAME \
                                | S3C2410_LCDCON5_HWSWP,
	},
	.lpcsel=	0x00000000,
	.gpccon=	0xaaaa02a8,
	.gpccon_mask=	0xffff03fc,
//	.gpcup=		0x0000ffff,
//	.gpcup_mask=	0xffffffff,
	.gpdcon=	0xaaaaaaaa,
	.gpdcon_mask=	0xffffffff,
//	.gpdup=		0x0000faff,
//	.gpdup_mask=	0xffffffff,

	.width=		S3C2410_WIDTH,
	.height=	S3C2410_HEIGHT,
	.xres=		{640,640,640},
	.yres=		{480,480,480},
	.bpp=		{16,16,16},
#endif
 /*
  *  LP35 screen (3.5 inch, 320x240) setup
  */
#ifdef CONFIG_LN2410SBC_LP35
#define S3C2410_WIDTH    320
#define S3C2410_HEIGHT   240
	.type=			S3C2410_LCDCON1_TFT,
	.regs={
		.lcdcon1 =   S3C2410_LCDCON1_TFT16BPP \
                                | S3C2410_LCDCON1_TFT \
                                | S3C2410_LCDCON1_CLKVAL(0x5),

		.lcdcon2 =  S3C2410_LCDCON2_VBPD(13) \
                                | S3C2410_LCDCON2_LINEVAL(S3C2410_HEIGHT-1) \
                                | S3C2410_LCDCON2_VFPD(22) \
				        | S3C2410_LCDCON2_VSPW(3),

		.lcdcon3 =  S3C2410_LCDCON3_HBPD(57) \
                                | S3C2410_LCDCON3_HOZVAL(S3C2410_WIDTH-1) \
				        | S3C2410_LCDCON3_HFPD(29),

 		.lcdcon4 =  S3C2410_LCDCON4_MVAL(13) \
				        | S3C2410_LCDCON4_HSPW(9),
                        

		.lcdcon5 =  S3C2410_LCDCON5_FRM565 \
                                | S3C2410_LCDCON5_INVVCLK \
                                | S3C2410_LCDCON5_INVVLINE \
                                | S3C2410_LCDCON5_INVVFRAME \
                                | S3C2410_LCDCON5_INVVD \
                                | S3C2410_LCDCON5_HWSWP,
	},
	.lpcsel=	0x00000000,
	.gpccon=	0xaaaa02a8,
	.gpccon_mask=	0xffff03fc,
//	.gpcup=		0x0000ffff,
//	.gpcup_mask=	0xffffffff,
	.gpdcon=	0xaaaaaaaa,
	.gpdcon_mask=	0xffffffff,
//	.gpdup=		0x0000faff,
//	.gpdup_mask=	0xffffffff,

	.width=		S3C2410_WIDTH,
	.height=	S3C2410_HEIGHT,
	.xres=		{320,320,320},
	.yres=		{240,240,240},
	.bpp=		{16,16,16},                                
#endif


};

static struct s3c24xx_mmc_platdata ln2410_mmc_cfg = {
	.gpio_detect	= S3C2410_GPF0,
	.set_power	= NULL,
	.f_max		= 1000000,
	.ocr_avail	= MMC_VDD_32_33,
	.f_min		= 100000,
	.bus_caps	= 0, //MMC_CAP_4_BIT_DATA,
};

static struct platform_device *ln2410sbc_devices[] __initdata = {
	&s3c_device_lcd,
	&s3c_device_adc,
	&s3c_device_wdt,
 	&s3c_device_rtc,
	&s3c_device_i2c,
	&s3c_device_nand,
	&s3c_device_usb,
	&s3c_device_usbgadget,
        &s3c_device_sdi,
	&s3c_device_ts,
	&s3c2410_device_buttons,
	&s3c_device_jsbuttons,
};

static struct s3c24xx_board ln2410sbc_board __initdata = {
	.devices       = ln2410sbc_devices,
	.devices_count = ARRAY_SIZE(ln2410sbc_devices)
};

static void __init ln2410sbc_map_io(void)
{
        s3c_device_nand.dev.platform_data = &ln2410_nand_info;

	s3c24xx_init_io(ln2410sbc_iodesc, ARRAY_SIZE(ln2410sbc_iodesc));
	s3c24xx_init_clocks(0);
	s3c24xx_init_uarts(ln2410sbc_uartcfgs, ARRAY_SIZE(ln2410sbc_uartcfgs));
	s3c24xx_set_board(&ln2410sbc_board);
}

static void __init ln2410sbc_init_irq(void)
{
	s3c24xx_init_irq();
}

static void __init ln2410_init(void)
{
	u32 tmp;
	s3c24xx_fb_set_platdata(&ln2410sbc_lcdcfg);
	set_s3c2410ts_info(&ln2410_ts_cfg);
	s3c_device_sdi.dev.platform_data = &ln2410_mmc_cfg;
 	s3c24xx_udc_set_platdata(&ln2410_udc_cfg);
	s3c2410_butt_set_platdata(&ln2410_butt_cfg);
	s3c24xx_jsbutt_set_platdata(&ln2410_jsbutt_cfg);
	/* Turn off suspend on both USB ports, and switch the
	 * selectable USB port to USB device mode. */
	s3c2410_gpio_cfgpin(S3C2410_GPE3,S3C2410_GPE3_OUTP);
	s3c2410_gpio_setpin(S3C2410_GPE3,1);
	s3c2410_modify_misccr(S3C2410_MISCCR_USBHOST |
			      S3C2410_MISCCR_USBSUSPND0 |
			      S3C2410_MISCCR_USBSUSPND1, 0x0);
	tmp = (
		 0x78 << S3C2410_PLLCON_MDIVSHIFT)
	      | (0x02 << S3C2410_PLLCON_PDIVSHIFT)
	      | (0x03 << S3C2410_PLLCON_SDIVSHIFT);
	writel(tmp, S3C2410_UPLLCON);
/* SD/MMC */
	s3c2410_gpio_pullup(S3C2410_GPE10,0);  //
        s3c2410_gpio_pullup(S3C2410_GPE9,0);   //
        s3c2410_gpio_pullup(S3C2410_GPE8,0);   //
        s3c2410_gpio_pullup(S3C2410_GPE7,0);   //
        s3c2410_gpio_pullup(S3C2410_GPE6,0);   //
    	s3c2410_gpio_pullup(S3C2410_GPE5,0);   //

}

MACHINE_START(LN2410SBC, "LN2410SBC") 
//	.phys_ram	= S3C2410_SDRAM_PA,
	.phys_io	= S3C2410_PA_UART,
	.io_pg_offst	= (((u32)S3C24XX_VA_UART) >> 18) & 0xfffc,
	.boot_params	= S3C2410_SDRAM_PA + 0x100,
	.map_io		= ln2410sbc_map_io,
	.init_irq	= ln2410sbc_init_irq,
	.timer		= &s3c24xx_timer,
	.init_machine   = ln2410_init,
MACHINE_END


