/* linux/drivers/ide/arm/ln2410sbc-ide-cf.c
 * Compact flash ide driver for LN2410SBC rev 1.6
 * Based on BAST IDE driver
 *
 * Copyright (c) 2003-2004 Simtec Electronics
 *  Ben Dooks <ben@simtec.co.uk>
 *  
 * 2006 Modified for LN2410SBC by ragrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

/* CF_MODE Selects Compact Flash mode 0 to 3
   0 -  Memory Mapped
   1 -  I/O Mapped 16 contiguous reqisters 0x00 - 0x0f
   2 -  Primary I/O Mapped 0x1f0 - 0x1f7,0x3f6-0x3f7
   3 -  Secondary I/O Mapped 0x1f0 - 0x1f7,0x3f6-0x3f7  
*/
#define CF_MODE 2

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/ide.h>
#include <linux/init.h>
#include <linux/delay.h>

#include <asm/mach-types.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/arch/map.h>
#include <asm/arch/ln2410sbc-map.h>
#include <asm/arch/irqs.h>
#include <asm/arch/hardware.h>
#include <asm/arch/regs-gpio.h>

/* list of registered interfaces */
static ide_hwif_t *ifsd[1];

static int __init
ln2410sbc_ide_cf_register(unsigned long base, unsigned long aux, int irq,
		 ide_hwif_t **hwif)
{
	hw_regs_t hw;
	int i;

	memset(&hw, 0, sizeof(hw));

	for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		hw.io_ports[i] = (unsigned long)base;
		base++;
	}

	hw.io_ports[IDE_CONTROL_OFFSET] = (unsigned long) aux;
	hw.irq = irq;
	ide_register_hw(&hw, hwif);

	return 0;
}


int cf_present(void)
{
   
     return !s3c2410_gpio_getpin(S3C2410_GPC7); 
 
}


static int __init ln2410sbc_ide_cf_init(void)
{
        unsigned long ide_base,ide_aux,conf_reg;
        int ide_irq;

	if (!(machine_is_ln2410sbc()))
		return 0;

	printk("LN2410_CF: CF driver, (c) 2003-2004 Simtec Electronics 2006 LAT\n");
	s3c2410_gpio_cfgpin(S3C2410_GPF1, S3C2410_GPF1_EINT1);
	printk("LN2410_CF:Configured pin GPF1 as Interrupt \n");
        s3c2410_gpio_cfgpin(S3C2410_GPC7, S3C2410_GPC7_INP);
	printk("LN2410_CF:Configured pin GPC7 as Input \n");
//	set_irq_type(IRQ_EINT1, IRQT_HIGH);
        if(cf_present());
        {
          conf_reg = (unsigned long)(LN2410SBC_VA_CF_ATTRIB_MEM + 0x200);
#if CF_MODE == 0
          printk("LN2410_CF:Configuring CF for Mode 0 - Memory Mapped \n");
          *(volatile char *)(conf_reg) = (((*(volatile char *) conf_reg)&(0xC0))|(0x0));
          ide_base = (unsigned long)(LN2410SBC_VA_CF_COMM_MEM);
          ide_aux = (unsigned long)(LN2410SBC_VA_CF_COMM_MEM + 0x0E);
#elif CF_MODE == 1
          printk("LN2410_CF:Configuring CF for Mode 1 - Direct IO \n");
          *(volatile char *)(conf_reg) = (((*(volatile char *) conf_reg)&(0xC0))|(0x1));
          ide_base = (unsigned long)(LN2410SBC_VA_CF_IO_MEM);
          ide_aux = (unsigned long)(LN2410SBC_VA_CF_IO_MEM + 0x0E);
#elif CF_MODE == 2
          printk("LN2410_CF:Configuring CF for Mode 2 - Primary IO \n");
          *(volatile char *)(conf_reg) = (((*(volatile char *) conf_reg)&(0xC0))|(0x2));
          ide_base = (unsigned long)(LN2410SBC_VA_CF_IO_MEM + 0x01f0);
          ide_aux = (unsigned long)(LN2410SBC_VA_CF_IO_MEM + 0x03f6);
#elif CF_MODE == 3
          printk("LN2410_CF:Configuring CF for Mode 3 - Secondary IO \n");
          *(volatile char *)(conf_reg) = (((*(volatile char *) conf_reg)&(0xC0))|(0x3));
          ide_base = (unsigned long)(LN2410SBC_VA_CF_IO_MEM + 0x0170);
          ide_aux = (unsigned long)(LN2410SBC_VA_CF_IO_MEM + 0x0376);
#endif
          ide_irq = IRQ_EINT1;
	  printk("LN2410_CF:Registering Ide\n");
	  ln2410sbc_ide_cf_register(ide_base, ide_aux, ide_irq, &ifsd[0]);
	  printk("LN2410_CF:Registered Ide\n");
	  return 0;
        }
	printk("LN2410_CF: No card is slot\n");
	return 1;
}

module_init(ln2410sbc_ide_cf_init);

MODULE_AUTHOR("Ben Dooks <ben@simtec.co.uk>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("LN2410SBC IDE Compact Flash driver");
