/* linux/drivers/ide/arm/ln2410sbc-ide-hdd.c
 * IDE Hard Disk Drive driver for LN2410SBC rev 1.6
 * Based on BAST IDE driver
 *
 * Copyright (c) 2003-2004 Simtec Electronics
 *  Ben Dooks <ben@simtec.co.uk>
 *  
 * Modified for LN2410SBC by ragrot
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/ide.h>
#include <linux/init.h>
#include <linux/delay.h>

#include <asm/mach-types.h>

#include <asm/io.h>
#include <linux/irq.h>
#include <asm/arch/map.h>
#include <asm/arch/ln2410sbc-map.h>
#include <asm/arch/irqs.h>
#include <asm/arch/hardware.h>
#include <asm/arch/regs-gpio.h>


/* list of registered interfaces */
static ide_hwif_t *ifs[1];

static int __init
ln2410sbc_ide_hdd_register(unsigned long base, unsigned long aux, int irq,
		 ide_hwif_t **hwif)
{
	hw_regs_t hw;
        unsigned long index; 
	int i;

	memset(&hw, 0, sizeof(hw));    
        index = base;
	for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		hw.io_ports[i] = (unsigned long)(index + 2*i);
	}
        for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		printk("Offset %d port: 0x%lx \n",i,hw.io_ports[i]);
	}
	hw.io_ports[IDE_CONTROL_OFFSET] = (unsigned long) aux;
	hw.irq = irq;
	ide_register_hw(&hw, hwif);

	return 0;
}


static int __init ln2410sbc_ide_hdd_init(void)
{
        unsigned long ide_base,ide_aux;
        int ide_irq;

	if (!(machine_is_ln2410sbc()))
		return 0;
	printk("LN2410_HDD: HDD driver, (c) 2003-2004 Simtec Electronics 2006 LAT\n");
        ide_base = (unsigned long)(LN2410SBC_VA_IDECMD);
        ide_aux = (unsigned long)(LN2410SBC_VA_IDECTL + (0x06)*2);

	s3c2410_gpio_cfgpin(S3C2410_GPG4, S3C2410_GPG4_EINT12);
        s3c2410_gpio_pullup(S3C2410_GPG4,0);
	printk("LN2410_HDD:Configured pin GPG4 as Interrupt with Pullup Resistor\n");
        ide_irq = IRQ_EINT12;
	set_irq_type(ide_irq, IRQF_TRIGGER_RISING);
	printk("LN2410_HDD:Registering Ide\n");
	ln2410sbc_ide_hdd_register(ide_base, ide_aux, ide_irq, &ifs[0]);
	printk("LN2410_HDD:Registered Ide\n");
        return 1;

}

module_init(ln2410sbc_ide_hdd_init);

MODULE_AUTHOR("Ben Dooks <ben@simtec.co.uk>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("LN2410SBC IDE HDD driver");
